summaryrefslogtreecommitdiffstats
path: root/drivers/misc/sgi-xp/xp_sn2.c
diff options
context:
space:
mode:
authorDean Nelson2008-07-30 07:34:16 +0200
committerLinus Torvalds2008-07-30 18:41:50 +0200
commita812dcc3a298eef650c381e094e2cf41a4ecc9ad (patch)
treecb6a24a623891054789f5340742c3c8ff1fb8cde /drivers/misc/sgi-xp/xp_sn2.c
parentsgi-xp: enable building of XPC/XPNET on x86_64 (diff)
downloadkernel-qcow2-linux-a812dcc3a298eef650c381e094e2cf41a4ecc9ad.tar.gz
kernel-qcow2-linux-a812dcc3a298eef650c381e094e2cf41a4ecc9ad.tar.xz
kernel-qcow2-linux-a812dcc3a298eef650c381e094e2cf41a4ecc9ad.zip
sgi-xp: add usage of GRU driver by xpc_remote_memcpy()
Add UV support to xpc_remote_memcpy(), which involves interfacing to the GRU driver. Signed-off-by: Dean Nelson <dcn@sgi.com> Cc: Jack Steiner <steiner@sgi.com> Cc: "Luck, Tony" <tony.luck@intel.com> Signed-off-by: Andrew Morton <akpm@linux-foundation.org> Signed-off-by: Linus Torvalds <torvalds@linux-foundation.org>
Diffstat (limited to 'drivers/misc/sgi-xp/xp_sn2.c')
-rw-r--r--drivers/misc/sgi-xp/xp_sn2.c50
1 files changed, 24 insertions, 26 deletions
diff --git a/drivers/misc/sgi-xp/xp_sn2.c b/drivers/misc/sgi-xp/xp_sn2.c
index c6a1ede7d6e6..1440134caf31 100644
--- a/drivers/misc/sgi-xp/xp_sn2.c
+++ b/drivers/misc/sgi-xp/xp_sn2.c
@@ -63,7 +63,7 @@ xp_register_nofault_code_sn2(void)
return xpSuccess;
}
-void
+static void
xp_unregister_nofault_code_sn2(void)
{
u64 func_addr = *(u64 *)xp_nofault_PIOR;
@@ -75,44 +75,41 @@ xp_unregister_nofault_code_sn2(void)
}
/*
+ * Convert a virtual memory address to a physical memory address.
+ */
+static unsigned long
+xp_pa_sn2(void *addr)
+{
+ return __pa(addr);
+}
+
+/*
* Wrapper for bte_copy().
*
* vdst - virtual address of the destination of the transfer.
* psrc - physical address of the source of the transfer.
* len - number of bytes to transfer from source to destination.
*
* Note: xp_remote_memcpy_sn2() should never be called while holding a spinlock.
*/
static enum xp_retval
-xp_remote_memcpy_sn2(void *vdst, const void *psrc, size_t len)
+xp_remote_memcpy_sn2(unsigned long dst_pa, const unsigned long src_pa,
+ size_t len)
{
bte_result_t ret;
- u64 pdst = ia64_tpa(vdst);
- /* ??? What are the rules governing the src and dst addresses passed in?
- * ??? Currently we're assuming that dst is a virtual address and src
- * ??? is a physical address, is this appropriate? Can we allow them to
- * ??? be whatever and we make the change here without damaging the
- * ??? addresses?
- */
- /*
- * Ensure that the physically mapped memory is contiguous.
- *
- * We do this by ensuring that the memory is from region 7 only.
- * If the need should arise to use memory from one of the other
- * regions, then modify the BUG_ON() statement to ensure that the
- * memory from that region is always physically contiguous.
- */
- BUG_ON(REGION_NUMBER(vdst) != RGN_KERNEL);
-
- ret = bte_copy((u64)psrc, pdst, len, (BTE_NOTIFY | BTE_WACQUIRE), NULL);
+ ret = bte_copy(src_pa, dst_pa, len, (BTE_NOTIFY | BTE_WACQUIRE), NULL);
if (ret == BTE_SUCCESS)
return xpSuccess;
- if (is_shub2())
- dev_err(xp, "bte_copy() on shub2 failed, error=0x%x\n", ret);
- else
- dev_err(xp, "bte_copy() failed, error=%d\n", ret);
+ if (is_shub2()) {
+ dev_err(xp, "bte_copy() on shub2 failed, error=0x%x dst_pa="
+ "0x%016lx src_pa=0x%016lx len=%ld\\n", ret, dst_pa,
+ src_pa, len);
+ } else {
+ dev_err(xp, "bte_copy() failed, error=%d dst_pa=0x%016lx "
+ "src_pa=0x%016lx len=%ld\\n", ret, dst_pa, src_pa, len);
+ }
return xpBteCopyError;
}
@@ -132,6 +129,7 @@ xp_init_sn2(void)
xp_partition_id = sn_partition_id;
xp_region_size = sn_region_size;
+ xp_pa = xp_pa_sn2;
xp_remote_memcpy = xp_remote_memcpy_sn2;
xp_cpu_to_nasid = xp_cpu_to_nasid_sn2;
@@ -75,44 +75,41 @@ xp_unregister_nofault_code_sn2(void)
* Wrapper for bte_copy().
*
- * vdst - virtual address of the destination of the transfer.
- * psrc - physical address of the source of the transfer.
+ * dst_pa - physical address of the destination of the transfer.
+ * src_pa - physical address of the source of the transfer.
* len - number of bytes to transfer from source to destination.
*